/***************************************************************************
 *   Copyright (C) 2004 by Georges A.K.                                    *
 *   georgesak@spymac.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <netdb.h>
#include <netinet/in.h>

#include <qsocketnotifier.h>

#include "kbeamobex.h"
#include "kbeamobexobject.h"


KBeamObex::KBeamObex()
{
	// Get a handle from OpenObex
	mHandle = OBEX_Init(OBEX_TRANS_IRDA, handleEventStatic, 
		OBEX_FL_KEEPSERVER);
	// Set our user data to this object
	OBEX_SetUserCallBack(mHandle, handleEventStatic, (void *)this);

	mConnected = false;
}

KBeamObex::KBeamObex(obex_t *pHandle)
{
	mHandle = pHandle;
	OBEX_SetUserCallBack(mHandle, handleEventStatic, (void *)this);
	createNotifier();

	mConnected = false;
}

KBeamObex::~KBeamObex()
{
	if (mConnected) {
		transportDisconnect();
	}
	deleteNotifier();
	cleanup();
}

void KBeamObex::registerServer()
{
	IrOBEX_ServerRegister(mHandle, "OBEX");
	createNotifier();
}

bool KBeamObex::transportConnect()
{
	if (IrOBEX_TransportConnect(mHandle, "OBEX") < 0) {
		return false;
	}

	createNotifier();

	mConnected = true;

	return true;
}

void KBeamObex::transportDisconnect()
{
	deleteNotifier();
	OBEX_TransportDisconnect(mHandle);
}

void KBeamObex::createNotifier()
{
	mNotifier = new QSocketNotifier(OBEX_GetFD(mHandle), 
		QSocketNotifier::Read);
	mNotifier->setEnabled(true);
	connect(mNotifier, SIGNAL(activated(int)), this, 
		SLOT(slotHandleInput(int)));
}

void KBeamObex::deleteNotifier()
{
	if (mNotifier) {
		mNotifier->setEnabled(false);
		delete mNotifier;
	}
}

void KBeamObex::pauseNotifier()
{
	if (mNotifier) {
		mNotifier->setEnabled(false);
	}
}

void KBeamObex::resumeNotifier()
{
	if (mNotifier) {
		mNotifier->setEnabled(true);
	}
}


void KBeamObex::request(KBeamObexObject &pObject)
{
	OBEX_Request(mHandle, pObject.mObject);
}

void KBeamObex::cancelRequest(int pNice)
{
	OBEX_CancelRequest(mHandle, pNice);
}

void KBeamObex::cleanup()
{
	if (mHandle) {
		OBEX_Cleanup(mHandle);
	}
}

obex_t *KBeamObex::accept() const
{
	obex_t *lHandle = OBEX_ServerAccept(mHandle, NULL, NULL);
	return  lHandle;
}

QString KBeamObex::responseToQString (KBeamObex::Response pRsp)
{
	char *lPtr = OBEX_ResponseToString(int(pRsp));

	QString lTmp (lPtr);

	return lTmp;
}

QByteArray* KBeamObex::qStringToUnicode(const QString &pSrc)
{
	const char *pOrig = pSrc.latin1();

	int lLength = 2*(pSrc.length() + 1);
	QByteArray *lTmp = new QByteArray(lLength);
	char *lTmpArray = lTmp->data();

	OBEX_CharToUnicode ((uint8_t *)lTmpArray, (uint8_t *)pOrig, lLength);

	return lTmp;
}

QString KBeamObex::obexstringToQString(unsigned short* pBuf)
{
	for (int lPos=0;pBuf[lPos]!=0;++lPos) {
		pBuf[lPos] = ntohs(pBuf[lPos]);
	}
	QString lTmp = QString::fromUcs2(pBuf);
	for (int lPos=0; pBuf[lPos]!=0; ++lPos) {
		pBuf[lPos] = htons(pBuf[lPos]);
	}
	return lTmp;
}

void KBeamObex::handleEvent(obex_object_t *pObject, int /*pMode*/, int pEvent,
	int pObexCmd, int pObexRsp)
{
	KBeamObexObject lObject(mHandle, pObject);

	switch (pEvent) {
	case OBEX_EV_PROGRESS:
		emit signalProgressMade(lObject);
		break;

	case OBEX_EV_ABORT:
		emit signalRequestAborted(lObject);
		break;

	case OBEX_EV_REQDONE:
		emit signalRequestDone(lObject, KBeamObex::Command(pObexCmd),
			KBeamObex::Response(pObexRsp));
		break;

	case OBEX_EV_REQHINT:
		emit signalRequestHint(lObject, KBeamObex::Command(pObexCmd));
		break;

	case OBEX_EV_REQ:
		emit signalRequest(lObject, KBeamObex::Command(pObexCmd));
		break;

	case OBEX_EV_LINKERR:
		emit signalLinkError ();
		break;

	case OBEX_EV_STREAMEMPTY:
		emit signalStreamEmpty(lObject);
		break;

	case OBEX_EV_STREAMAVAIL:
		emit signalStreamAvailble(lObject);
		break;

	case OBEX_EV_ACCEPTHINT:
		emit signalAcceptHint(*this, lObject);
		break;

	default:
		break;
	}

}

void KBeamObex::handleEventStatic(obex_t *pHandle, obex_object_t *pObject, 
	int pMode, int pEvent, int pObexCmd, int pObexRsp)
{
	KBeamObex *lObex = (KBeamObex *)OBEX_GetUserData(pHandle);
	lObex->handleEvent(pObject, pMode, pEvent, pObexCmd, pObexRsp);
}

void KBeamObex::slotHandleInput(int)
{
	if (OBEX_HandleInput (mHandle, 0) < 0) {
		emit signalLinkError();
	}
}
