/***************************************************************************
 *   Copyright (C) 2004 by Georges A.K.                                    *
 *   georgesak@spymac.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "kbeamsend.h"
#include "kbeamobexobject.h"
#include "kbeamobexheader.h"
#include "kbeamevent.h"
#include "kbeamconfig.h"

#include <qfileinfo.h>
#include <qfile.h>
#include <qtimer.h>

KBeamSend::KBeamSend(const QString &pFilename) :
	KProgressDialog(NULL, NULL, "KBeam", "Initiating file transfer")
{
	mFilename = pFilename;
	showCancelButton(true);
	disableResize();
	setAutoClose(false);
	progressBar()->setMinimumSize(300, 25);
	hide();
}

KBeamSend::~ KBeamSend()
{
	if (mObex) {
		delete mObex;
	}
}

void KBeamSend::start()
{
	mObex = new KBeamObex();

	connect(mObex, SIGNAL(signalStreamEmpty(KBeamObexObject& )), this,
		SLOT(slotStreamEmpty(KBeamObexObject& )));
	connect(mObex, SIGNAL(signalRequestDone(KBeamObexObject&, 
		KBeamObex::Command, KBeamObex::Response )), this,
		SLOT(slotRequestDone(KBeamObexObject&, KBeamObex::Command, 
		KBeamObex::Response )));
	connect(mObex, SIGNAL(signalRequestAborted(KBeamObexObject &)), this,
		SLOT(slotRequestAborted(KBeamObexObject &)));
	connect(mObex, SIGNAL(signalLinkError()), this,
		SLOT(slotLinkError()));

	connect(&mLinkTimer, SIGNAL(timeout()), this, SLOT(slotLinkTimeout()));

	// Send event that we're sending file
	KBeamEvent::sendStartSend(mFilename);

	if (KBeamConfig::self()->showSendDlg()) {
		show();
	} else {
		hide();
	}

	if (!mObex->transportConnect()) {
		aborted("Could not initiate file transfer (couldn't connect to remote \
device)");
	} else {
		KBeamObexObject lObject(*mObex, KBeamObex::Connect);

		const char *lWho = "Linux";
		KBeamObexHeader lWhoHeader(KBeamObexHeader::Who, (uint8_t *) lWho, 6);
		lObject.addHeader(lWhoHeader, KBeamObexObject::FitOnePacket);

		mObex->request(lObject);
	}

}

void KBeamSend::slotRequestDone(KBeamObexObject &pObject, KBeamObex::Command pCmd,
	KBeamObex::Response pRsp)
{
	switch (pCmd) {
	case KBeamObex::Connect:
		if (pRsp == KBeamObex::Success) {
			pObject.setResponse(KBeamObex::Success, KBeamObex::Success);
			QTimer::singleShot(0, this, SLOT (slotStartSend()));
		} else {
			aborted("Could not initiate file transfer (couldn't connect to \
remote device)");
		}
		break;
	case KBeamObex::Disconnect:
		break;
	case KBeamObex::Put:
		if (pRsp == KBeamObex::Success) {
			done();
		}
		break;
	default:
		break;
	}
}

void KBeamSend::slotStartSend()
{
	QFileInfo lFileInfo(mFilename);
	QString lTarget = lFileInfo.fileName();

	QFile lFile(mFilename);
	if (!lFile.open (IO_ReadOnly)) {
		aborted("Could not open file for reading !");
		return;
	}

	mBuffer = lFile.readAll();

	lFile.close ();

	setLabel(QString("Sending file : ") + lTarget + QString(" (") + 
		QString::number(mBuffer.size()) + QString(" bytes)"));

	progressBar()->setTotalSteps(mBuffer.size());

	KBeamObexObject lObject(*mObex, KBeamObex::Put);

	QByteArray *lUnicode = KBeamObex::qStringToUnicode(lTarget);
	KBeamObexHeader lNameHeader(KBeamObexHeader::Name, 
		(uint8_t *) lUnicode->data(), lUnicode->size());
	lObject.addHeader (lNameHeader, KBeamObexObject::None);
	delete lUnicode;

	KBeamObexHeader lLengthHeader (KBeamObexHeader::Length, 
		(uint32_t)mBuffer.size(), 4);
	lObject.addHeader(lLengthHeader, KBeamObexObject::None);

	KBeamObexHeader lStreamHeader (KBeamObexHeader::Body, (uint8_t *)NULL, 0);
	lObject.addHeader(lStreamHeader, KBeamObexObject::StreamStart);

	mCurrentSize = 0;

	mObex->request(lObject);
}

void KBeamSend::slotStreamEmpty(KBeamObexObject &pObject)
{
	const int lStreamChunk = 512;

	if (isShown() && !KBeamConfig::self()->showSendDlg()) {
		hide();
	}

	// Stop the link timer
	mLinkTimer.stop();

	progressBar()->setProgress(mCurrentSize);
	setCaption(QString::number(mCurrentSize*100/mBuffer.size()) + 
		QString("%"));

	if (mCurrentSize == mBuffer.size()) {
		KBeamObexHeader lEndHeader (KBeamObexHeader::Body,
			(uint8_t *) mBuffer.data() + mCurrentSize, 0);
		pObject.addHeader (lEndHeader, KBeamObexObject::StreamEnd);
	} else {
		int lRemain = mBuffer.size() - mCurrentSize;
		if (lRemain < lStreamChunk) {
			KBeamObexHeader lDataHeader(KBeamObexHeader::Body,
				(uint8_t *)(mBuffer.data() + mCurrentSize), lRemain);
			pObject.addHeader (lDataHeader, KBeamObexObject::StreamData);
			mCurrentSize += lRemain;
		} else {
			KBeamObexHeader lDataHeader(KBeamObexHeader::Body,
				(uint8_t *)(mBuffer.data() + mCurrentSize), lStreamChunk);
			pObject.addHeader(lDataHeader, KBeamObexObject::StreamData);
			mCurrentSize += lStreamChunk;
		}
	}

	// Start the link timer
	mLinkTimer.start(2000, true);
}

void KBeamSend::slotLinkError()
{
	aborted("File transfer was aborted because the link was broken");
}

void KBeamSend::slotRequestAborted(KBeamObexObject &/*pObject*/)
{
	aborted("File transfer was aborted by remote device");
}

void KBeamSend::closeEvent(QCloseEvent */*pEvent*/)
{
	slotCancel();
}

void KBeamSend::slotCancel()
{
	delete this;
}

void KBeamSend::slotLinkTimeout()
{
	KBeamEvent::sendLinkInterrupted();
}

void KBeamSend::done()
{
	mLinkTimer.stop();
	mObex->pauseNotifier();
	setCaption(QString("Done"));
	setButtonText(QString("&Close"));
	KBeamEvent::sendFileSent(mFilename);
	if (isHidden() || KBeamConfig::self()->autoCloseSendDlg()) {
		QTimer::singleShot(1000, this, SLOT (slotCancel()));
	}
}

void KBeamSend::aborted(const QString &pReason)
{
	mLinkTimer.stop();
	mObex->pauseNotifier();
	setCaption(QString("Error"));
	progressBar()->setEnabled(false);
	setLabel(pReason);
	setButtonText(QString("&Close"));
	KBeamEvent::sendSendAborted(mFilename);
	if (isHidden()) {
		QTimer::singleShot(1000, this, SLOT (slotCancel()));
	}
}
