/***************************************************************************
 *   Copyright (C) 2004 by Georges A.K.                                    *
 *   georgesak@spymac.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include <sys/socket.h>
#include <linux/types.h>
#include <linux/irda.h>

#include <krun.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <qtimer.h>
#include <qfile.h>

#include "kbeamsystem.h"
#include "kbeamconfig.h"
#include "kbeamreceive.h"
#include "kbeamsend.h"
#include "kbeamobex.h"
#include "kbeamobexobject.h"


KBeamSystem::KBeamSystem()
{
}

KBeamSystem::~KBeamSystem()
{
	stopServer();
	close(mFd);
}
 
bool KBeamSystem::init()
{
	unsigned char lHints[4];

	checksSettings();

	// Open IRDA socket
	mFd = socket(AF_IRDA, SOCK_STREAM, 0);
	if (mFd == 0) {
		return false;
	}

	lHints[0] = HINT_EXTENSION;
	lHints[1] = HINT_OBEX;

	// Set the socket options for discovery
	if (setsockopt(mFd, SOL_IRLMP, IRLMP_HINT_MASK_SET,
					lHints, sizeof(lHints))) {
		return false;
	}

	// Start the server
	startServer();

	// Start discovering
	QTimer::singleShot(1000, this, SLOT(slotDiscover()));

	return true;
}

void KBeamSystem::sendFile(const QString pFilename)
{
	KBeamSend *lSend = new KBeamSend(pFilename);
	lSend->start();
}

void KBeamSystem::slotDiscover()
{
	irda_device_list *lList;
	int lLen;

	// Prepare necessary structures
	// TODO: Make sMaxDevices configurable
	lLen = sizeof(irda_device_list) + sizeof(irda_device_info) * 10/*sMaxDevices*/;
	lList = (struct irda_device_list *)new char[lLen];
	memset((char *)lList, 0, lLen);

	// Discover devices
	getsockopt(mFd, SOL_IRLMP, IRLMP_ENUMDEVICES, (unsigned char *)lList, 
					(socklen_t*)&lLen);

	// Update device list
	mDeviceList.updateDevices(lList);

	// Delete allocated structure
	delete lList;

	// Schedule another discovery in 1 sec
	QTimer::singleShot(1000, this, SLOT(slotDiscover()));
}

void KBeamSystem::startServer()
{
	// Create a new obex handle and register a server
	mObexServer = new KBeamObex();
	mObexServer->registerServer();
	// Monitor the Accept Hint signal (which means an incoming request)
	connect(mObexServer, SIGNAL(signalAcceptHint(KBeamObex &, 
		KBeamObexObject &)), this, SLOT(slotAcceptHint(KBeamObex &, 
		KBeamObexObject &)));
}

void KBeamSystem::stopServer()
{
	delete mObexServer;
}

void KBeamSystem::checksSettings()
{
	QByteArray lBuffer;
	QFile lFile("/proc/sys/net/irda/max_baud_rate");

	if (!lFile.open(IO_ReadOnly)) {
		return;
	}
	lBuffer = lFile.readAll();
	int lBaudRate = QString(lBuffer).toInt();

	if (lBaudRate > 115200) {
		if (KMessageBox::questionYesNo(0, i18n("The baud rate on this system \
is set to a value higher than 115200. This may cause problems with certain \
devices. \n Do you want to correct this ? By pressing Yes, you will be \
prompted your root password, since this action requires root privilages. you \
may have to restart your irda stack and KBeam for the change to take effect. \
Please consult the README file for more details."), 
			i18n("Baud rate higher than 115200")) == KMessageBox::Yes) {
				KRun::runCommand(
					"kdesu \"echo 115200 > /proc/sys/net/irda/max_baud_rate\"");
		}
	}
}

void KBeamSystem::slotAcceptHint(KBeamObex &pObex, KBeamObexObject &/*pObject*/)
{
	KBeamReceive *lReceive = new KBeamReceive();
	lReceive->receive(pObex);
}
